# %%
from regmonkey_style.config import CONFIG


class Templates:
    @classmethod
    def regmonkey_scatter(cls):
        custom_template = dict(
            {
                "data": {
                    "scatter": [
                        {
                            "mode": "markers",
                            "opacity": CONFIG.scatter.opacity,
                            "marker": {"size": CONFIG.scatter.markersize.plotly},
                            "line": {"width": CONFIG.common.line_width},
                        }
                    ]
                },
                "layout": cls._create_layout_template(
                    CONFIG.color_style.qualitative_scatter_color
                ),
            }
        )

        return custom_template

    @classmethod
    def regmonkey_line(cls):
        custom_template = dict(
            {
                "data": {
                    "scatter": [
                        {
                            "mode": "lines",
                            "opacity": CONFIG.line.opacity,
                            "marker": {"size": CONFIG.scatter.markersize.plotly},
                            "line": {"width": CONFIG.common.line_width},
                            "connectgaps": True,
                        }
                    ]
                },
                "layout": cls._create_layout_template(
                    CONFIG.color_style.qualitative_line_color
                ),
            }
        )

        return custom_template

    @classmethod
    def regmonkey_twoline(cls):
        custom_template = dict(
            {
                "data": {
                    "scatter": [
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "solid"},
                            "connectgaps": True,
                        },
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "solid"},
                            "connectgaps": True,
                        },
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "solid"},
                            "connectgaps": True,
                        },
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "dash"},
                            "connectgaps": True,
                        },
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "dash"},
                            "connectgaps": True,
                        },
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "dot"},
                            "connectgaps": True,
                        },
                        {
                            "mode": "markers+lines",
                            "line": {"dash": "dot"},
                            "connectgaps": True,
                        },
                    ]
                },
                "layout": cls._create_layout_template(
                    CONFIG.color_style.two_line_color
                ),
            }
        )

        return custom_template

    @classmethod
    def regmonkey_boxplot(cls):
        custom_layout_template = cls._create_layout_template(
            CONFIG.color_style.two_line_color
        )
        # Define the layout as a dictionary
        custom_layout_template.update(
            {
                "boxmode": "group",  # Group boxes for comparison
                "plot_bgcolor": CONFIG.color_style.background_color,
                "margin": CONFIG.boxplot.margin.plotly,
                "font": {
                    "size": CONFIG.boxplot.fontsize.text_fontsize,
                    "color": "black",
                },
                "xaxis": {
                    "title": {
                        "text": "X-axis Title",
                        "standoff": CONFIG.common.fontsize.standoff,
                    },
                    "titlefont": {"size": CONFIG.boxplot.fontsize.xlabel_fontsize},
                    "tickfont": {"size": CONFIG.boxplot.fontsize.xlabel_fontsize},
                    "showgrid": False,
                    "showline": True,
                    "zeroline": False,
                },
                "yaxis": {
                    "title": {
                        "text": "Y-axis Title",
                        "standoff": CONFIG.common.fontsize.standoff,
                    },
                    "titlefont": {"size": CONFIG.boxplot.fontsize.ylabel_fontsize},
                    "tickfont": {"size": CONFIG.boxplot.fontsize.tick_fontsize},
                    # "scaleanchor": "x",  # Lock y-axis scale to x-axis
                    # "scaleratio": 1,  # Ensure equal scale ratio
                    "showgrid": True,
                    "showline": False,
                    "zeroline": False,
                    "griddash": CONFIG.boxplot.gridline.griddash.plotly,
                    "gridcolor": CONFIG.color_style.grid_color,
                    "gridwidth": CONFIG.boxplot.gridline.gridwidth,
                },
                "legend": {
                    "font": {"size": CONFIG.common.fontsize.tick_fontsize},
                    "bgcolor": CONFIG.color_style.legend_background_color,
                    "traceorder": "normal",
                },
            }
        )
        # Create a template with the layout
        custom_template = dict(
            layout=custom_layout_template,
            # data={
            #     "box":[  # The box property should be a list
            #         dict(
            #             jitter=CONFIG.boxplot.jitter,  # Jitter applied globally
            #             marker=dict(
            #                 color=CONFIG.boxplot.outliercolor,
            #                 size=CONFIG.boxplot.markersize.plotly,
            #                 outliercolor=CONFIG.boxplot.outliercolor,
            #                 line=dict(outlierwidth=0),
            #             ),
            #             line=dict(color="black", width=1.3),
            #             fillcolor="white",
            #             width=0.5,
            #             boxpoints="suspectedoutliers",  # Show suspected outliers
            #         ),
            #     ]
            # },
        )

        return custom_template

    @classmethod
    def _create_layout_template(cls, color_way):
        """
        Create a default layout template for Plotly.

        Args:
            color_way (list): List of colors to be used in the color cycle.

        Returns:
            dict: A dictionary representing the default layout template.
        """

        default_layout = dict(
            {
                "plot_bgcolor": "#EFF5F5",
                "margin": CONFIG.common.margin.plotly,
                "shapes": [],  # You can add shapes here if needed
                "paper_bgcolor": CONFIG.color_style.paper_bgcolor,
                "colorway": color_way,  # Color cycle for lines
                "font": {
                    "size": CONFIG.common.fontsize.text_fontsize,
                    "color": CONFIG.color_style.text_color,
                },
                "title": {
                    "font": {"size": CONFIG.common.fontsize.title_fontsize},
                    "x": 0.08,  # Horizontal position (left-aligned),
                    "yanchor": "top",
                },
                "autosize": True,
                "legend": {
                    "font": {"size": CONFIG.common.fontsize.tick_fontsize},
                    "bgcolor": CONFIG.color_style.legend_background_color,
                    "traceorder": "normal",
                },
                "xaxis": {
                    "title": {
                        "text": "X-axis Title",
                        "standoff": CONFIG.common.fontsize.standoff,
                    },
                    "titlefont": {"size": CONFIG.common.fontsize.xlabel_fontsize},
                    "tickfont": {"size": CONFIG.common.fontsize.tick_fontsize},
                    "showgrid": True,
                    "showline": True,
                    "griddash": "dot",
                    "gridcolor": CONFIG.color_style.grid_color,
                    "gridwidth": CONFIG.common.gridline.gridwidth,
                    "zeroline": True,
                    "zerolinecolor": CONFIG.color_style.zeroline_color,
                    "zerolinewidth": CONFIG.common.gridline.gridwidth,
                },
                "yaxis": {
                    "title": {
                        "text": "Y-axis Title",
                        "standoff": CONFIG.common.fontsize.standoff,
                    },
                    "titlefont": {"size": CONFIG.common.fontsize.ylabel_fontsize},
                    "tickfont": {"size": CONFIG.common.fontsize.tick_fontsize},
                    # "scaleanchor": "x",  # Lock y-axis scale to x-axis
                    # "scaleratio": 1,  # Ensure equal scale ratio
                    "showgrid": True,
                    "showline": True,
                    "griddash": "dot",
                    "gridcolor": CONFIG.color_style.grid_color,
                    "gridwidth": CONFIG.common.gridline.gridwidth,
                    "zeroline": True,
                    "zerolinecolor": CONFIG.color_style.zeroline_color,
                    "zerolinewidth": CONFIG.common.gridline.gridwidth,
                },
            }
        )
        return default_layout



# %%

from regmonkey_style.config import CONFIG
import matplotlib.pyplot as plt


def add_transparency(colors, alpha):
    """
    Add transparency to a list of colors in #RRGGBB format.

    Args:
        colors (list): List of color hex codes in #RRGGBB format.
        alpha (float): Transparency level (0.0 to 1.0).

    Returns:
        list: List of colors with added transparency in #RRGGBBAA format.
    """

    def hex_with_alpha(color, alpha):
        """
        Add alpha transparency to a single hex color.

        Args:
            color (str): Color hex code in #RRGGBB format.
            alpha (float): Transparency level (0.0 to 1.0).

        Returns:
            str: Color hex code in #RRGGBBAA format.
        """
        alpha_hex = f"{int(alpha * 255):02x}"
        return color + alpha_hex

    return [hex_with_alpha(color, alpha) for color in colors]


class Templates:
    @classmethod
    def regmonkey_scatter(cls):
        custom_colorway = plt.cycler(
            color=add_transparency(
                CONFIG.color_style.qualitative_scatter_color, CONFIG.scatter.opacity
            )
        )
        custom_layout_template = cls._create_layout_template(custom_colorway)
        custom_linewidth_template = cls._create_gridline_template(
            CONFIG.common.gridline.gridwidth * 4,
            CONFIG.common.gridline.gridwidth * 10,
        )

        custom_template = custom_layout_template | custom_linewidth_template

        ## update
        scatter_custom = dict(
            {"lines.markersize": CONFIG.scatter.markersize.matplotlib}
        )

        custom_template.update(scatter_custom)
        return custom_template

    @classmethod
    def regmonkey_line(cls):
        custom_colorway = plt.cycler(
            color=add_transparency(
                CONFIG.color_style.qualitative_line_color, CONFIG.line.opacity
            )
        )
        custom_layout_template = cls._create_layout_template(custom_colorway)
        custom_linewidth_template = cls._create_gridline_template(
            CONFIG.common.gridline.gridwidth * 4,
            CONFIG.common.gridline.gridwidth * 10,
        )

        custom_template = custom_layout_template | custom_linewidth_template

        return custom_template

    @classmethod
    def regmonkey_twoline(cls):
        custom_colorway = plt.cycler(
            color=CONFIG.color_style.two_line_color
        ) + plt.cycler(linestyle=["-", "-", "-", "--", "--", ":", ":"])

        custom_layout_template = cls._create_layout_template(custom_colorway)
        custom_linewidth_template = cls._create_gridline_template(
            CONFIG.common.gridline.gridwidth * 8,
            CONFIG.common.gridline.gridwidth * 10,
        )

        custom_template = custom_layout_template | custom_linewidth_template

        return custom_template

    @classmethod
    def regmonkey_boxplot(cls):
        custom_colorway = plt.cycler(
            color=add_transparency(
                CONFIG.color_style.two_line_color, 0.8
            )
        )

        custom_layout_template = cls._create_layout_template(custom_colorway)
        custom_linewidth_template = cls._create_gridline_template(
            CONFIG.common.gridline.gridwidth * 8,
            CONFIG.common.gridline.gridwidth * 10,
        )

        custom_boxplot_template = dict(
            {
                "axes.spines.left": False,
                "boxplot.medianprops.color": CONFIG.boxplot.line_color,
                "boxplot.flierprops.markersize": CONFIG.boxplot.markersize.matplotlib,
                "boxplot.flierprops.markeredgecolor": CONFIG.boxplot.outliercolor,
                "patch.facecolor": "white",
                "boxplot.patchartist": True,
                "patch.force_edgecolor": True,
                "boxplot.capprops.color": CONFIG.boxplot.line_color,
                "boxplot.boxprops.color": CONFIG.boxplot.line_color,
            }
        )

        custom_template = custom_layout_template | custom_linewidth_template
        custom_template.update(custom_boxplot_template)

        return custom_template

    @classmethod
    def _create_layout_template(cls, color_way):
        matplotlib_plotly_ratio = 1.3
        default_layout = dict(
            {
                "figure.facecolor": CONFIG.color_style.background_color,
                "axes.facecolor": CONFIG.color_style.background_color,
                "axes.edgecolor": "black",
                "font.size": CONFIG.common.fontsize.text_fontsize
                / matplotlib_plotly_ratio,
                "axes.titlelocation": "left",
                "axes.titlesize": CONFIG.common.fontsize.title_fontsize
                / (matplotlib_plotly_ratio * 1.1),
                "axes.labelsize": CONFIG.common.fontsize.xlabel_fontsize
                / matplotlib_plotly_ratio,
                "xtick.labelsize": CONFIG.common.fontsize.xlabel_fontsize
                / matplotlib_plotly_ratio,
                "ytick.labelsize": CONFIG.common.fontsize.tick_fontsize
                / matplotlib_plotly_ratio,
                "legend.fontsize": CONFIG.common.fontsize.legend_fontsize
                / matplotlib_plotly_ratio,
                "legend.frameon": False,
                "legend.loc": "upper right",  # Change location to upper left
                "axes.grid": True,
                "grid.linestyle": ":",
                "grid.color": CONFIG.color_style.grid_color,
                "axes.spines.top": False,
                "axes.spines.right": False,
                "axes.spines.left": True,
                "axes.spines.bottom": True,
                "axes.prop_cycle": color_way,
                "figure.subplot.wspace": 20,
                "text.color": CONFIG.color_style.text_color,  # Set text color
                "axes.labelcolor": CONFIG.color_style.text_color,  # Set axes label color
                "xtick.color": CONFIG.color_style.text_color,  # Set x-tick color
                "ytick.color": CONFIG.color_style.text_color,  # Set y-tick color
                "axes.titlecolor": CONFIG.color_style.text_color,  # Set axes title color
            }
        )
        return default_layout

    @classmethod
    def _create_gridline_template(cls, gridline_width, axesline_width):
        default_layout = dict(
            {
                "grid.linewidth": gridline_width,
                "axes.linewidth": axesline_width,
            }
        )
        return default_layout



# %%
from regmonkey_style.config import CONFIG
import plotly.io as pio
import matplotlib.pyplot as plt
import seaborn as sns
import regmonkey_style.common.setup_custom_font as scf
import regmonkey_style.plotly_custom.templates as pct
import regmonkey_style.matplotlib_custom.templates as mct
from regmonkey_style.common import utils


def show_available_fonts():
    return CONFIG.font_style.available_list


def show_available_templates():
    return CONFIG.templates


def set_font(font=CONFIG.font_style.default_font):
    if font not in CONFIG.font_style.available_list:
        raise ValueError("{} is not in templates list".format(font))
    scf.add_custom_font(font)


def set_templates(template):
    """
    Set the Plotly/Matplotlib template for the application.

    Args:
        template (str): The name of the template to set.

    Raises:
        ValueError: If the specified template is not in the list of available templates.
    """
    if template not in CONFIG.templates:
        raise ValueError(f"{template} is not in the templates list")

    # Set Plotly template
    __set_plotly_template(template)

    # Set Matplotlib template
    __set_matplotlib_template(template)


def __set_plotly_template(template):
    """
    Set the Plotly template for the application.

    Args:
        template (str): The name of the template to set.
    """
    plotly_template_instance = pct.Templates
    plotly_custom_template = getattr(plotly_template_instance, template)()
    pio.templates[template] = plotly_custom_template
    pio.templates.default = template


def __set_matplotlib_template(template):
    """
    Set the Matplotlib template for the application.

    Args:
        template (str): The name of the template to set.
    """
    matplotlib_template_instance = mct.Templates
    matplotlib_custom_template = getattr(matplotlib_template_instance, template)()
    sns.set_style(rc=matplotlib_custom_template)
    plt.rcParams.update(matplotlib_custom_template)


def equal_xy_scale(figure_object):
    """
    Ensure equal scaling for the x and y axes of a Plotly figure.

    Args:
        figure_object: The Plotly or Matplotlib figure object to modify.

    Returns:
        figure_object: The modified Plotly figure object with equal x and y scaling.
    """
    return utils.equal_xy_scale(figure_object)


def restore_default():
    """
    Restore the default Plotly template and Matplotlib settings.

    This function resets the Plotly template to the default "plotly" template
    and restores the default Matplotlib settings.
    """
    pio.templates.default = "plotly"
    plt.rcdefaults()



import os
import matplotlib
from matplotlib import font_manager
import plotly.io as pio

FONTS_DIR = "../fonts/"


def get_font_path(PATH):
    return os.path.abspath(os.path.join(os.path.dirname(__file__), FONTS_DIR + PATH))


def add_custom_font(FONT_NAME: str) -> None:
    # setup
    current_template = pio.templates.default
    font_dirs = [get_font_path(FONT_NAME)]
    font_files = font_manager.findSystemFonts(fontpaths=font_dirs)
    for f in font_files:
        font_manager.fontManager.addfont(f)
        pio.templates[current_template]["layout"]["font"] = dict(family=f)

    SYSTEM_FONT_NAME = font_manager.FontProperties(fname=font_files[0]).get_name()
    matplotlib.rcParams["font.family"] = SYSTEM_FONT_NAME
    pio.templates[current_template]["layout"]["font"] = dict(family=SYSTEM_FONT_NAME)



# %%
import pathlib
import yaml
from pydantic import BaseModel


class ColorStyle(BaseModel):
    qualitative_scatter_color: list[str]
    qualitative_line_color: list[str]
    discrete_color_sequence: list[str]
    two_line_color: list[str]
    background_color: str
    paper_bgcolor: str
    text_color: str
    legend_background_color: str
    grid_color: str
    zeroline_color: str


class FontStyle(BaseModel):
    default_font: str
    available_list: list[str]


class Margin(BaseModel):
    plotly: dict


class Griddash(BaseModel):
    plotly: str
    matplotlib: str


class Gridline(BaseModel):
    gridwidth: float
    griddash: Griddash


class Markersize(BaseModel):
    matplotlib: int
    plotly: int


class Fontsize(BaseModel):
    title_fontsize: int
    xlabel_fontsize: int
    ylabel_fontsize: int
    legend_fontsize: int
    text_fontsize: int
    tick_fontsize: int
    standoff: int


class Common(BaseModel):
    fontsize: Fontsize
    margin: Margin
    gridline: Gridline
    line_width: int


class Scatter(BaseModel):
    opacity: float
    markersize: Markersize


class Line(BaseModel):
    opacity: float


class Boxplot(BaseModel):
    opacity: float
    outliercolor: str
    line_color: str
    fillcolor: str
    jitter: float
    markersize: Markersize
    fontsize: Fontsize
    linewdith: float
    gridline: Gridline
    margin: Margin


class Config(BaseModel):
    """Config is a model for the configuration file of the project.
    This model should be generated from new_config function.
    """

    color_style: ColorStyle
    font_style: FontStyle
    common: Common
    scatter: Scatter
    line: Line
    boxplot: Boxplot
    templates: list[str]


def new_config(path_to_config: pathlib.Path) -> Config:
    """Generates a Config object from a yaml file.

    Args:
        path_to_config (str): Path to the configuration file.

    Returns:
        Config: A Config object.

    Example:
        >>> new_config(
        ...     "../configs/config.yaml"
        ... )
        Config(discrete_color_sequence=["#0E3666", "#428CE6", "#B4D7FF", "#E8F1FE"])
    """

    with open(path_to_config) as f:
        yaml_config = yaml.safe_load(f)
    config: Config = Config.model_validate(yaml_config)

    return config


PATH_TO_CONFIG: pathlib.Path = (
    pathlib.Path(__file__).parent / "configs/config.yaml"
)  # For package use, use __file__ instead of relative path
CONFIG: Config = new_config(path_to_config=PATH_TO_CONFIG)



import plotly.graph_objs as go
import matplotlib.pyplot as plt


def equal_xy_scale(fig_object):
    """
    Ensure equal scale on both x and y axes for Matplotlib figures.

    This function checks if the provided object is a Matplotlib `Axes` or `Figure`,
    and applies equal scaling to all the subplots.

    Parameters:
    fig_object : Matplotlib `Axes` or `Figure` object
        The figure or axes to apply equal scaling to.
    """
    if isinstance(fig_object, go.Figure):
        fig_object.update_layout(
            xaxis=dict(
                scaleanchor="y",  # Lock x-axis scale to y-axis
                scaleratio=1,  # Ensure equal scale ratio
            ),
            yaxis=dict(
                scaleanchor="x",  # Lock y-axis scale to x-axis
                scaleratio=1,  # Ensure equal scale ratio
            ),
        )
    elif isinstance(fig_object, plt.Axes):  # Check if it's an Axes object
        fig_object.set_aspect("equal", "box")
    elif isinstance(fig_object, plt.Figure):  # If it's a Figure, apply to all axes
        for ax in fig_object.get_axes():
            ax.set_aspect("equal", "box")
    else:
        raise TypeError("The object is not a valid Matplotlib Figure or Axes")

    return fig_object
